function [WhitenD pval] = WhitenTimeSeriesWrapper(SampleD, varargin)
% preWhiten implements a filter (yule-walker) to decorrelate the time series data. 
% Arguments
% 	SampleD	- Data Matrix can be timeseries x roi | timeseries x roi x subject
% 	L	- (Optional) Number of lags or the order of the AR model. 
% 		  Default assumes AR(L=3) model. 
% Outputs
% 	WhitenD	- Returns whitened data matrix of the same size as input

if(nargin==2)
	L = varargin{1}; 
else
	L = 3;
end
	
	[n p s] = size(SampleD);
	SampleD = reshape(SampleD, [n p*s]);
	WhitenD = zeros(size(SampleD));
	pval = zeros(p*s,1);

	parfor i=1:p*s	
       	[res pval(i) ab acf] = Whiten(zscore(SampleD(:,i)),L,0);
		WhitenD(:,i) = zscore(res);
	end
	
	%%% Checking for dependency in samples
	%disp('No. dependent');
    %sum(pval)
	WhitenD = reshape(WhitenD, [n p s]);	
	
end


function [res H  ab acf] = Whiten(D,L,onlyTest)
%% Test and correct for autoregressive noise. Tested using Ljung Box test.
%% Inputs
%	- D is a vector or matrix of p x n where n is number of samples (correlated) and p is number of variables.
% 	- L is the number of autocorrelation lags assumed.
% 	- Test D for non-dependent samples without fitting an AR model; i.e. to check moving block subsamples.
%% Outputs
%	- corSamples - corrected samples that are no longer dependent over time
% 	- pValue of the resulting Ljung Box tests.
% [h,pValue,stat,cValue] = lbqtest(res) and [h,pValue,stat,cValue] = lbqtest(res,Name,Value) perform the Ljung-Box 
% lack-of-fit hypothesis test for model misspecification.
% 
% The "portmanteau" test of Ljung and Box assesses the null hypothesis that a series of residuals exhibits ...
% no autocorrelation for a fixed number of lags L, against the alternative that some autocorrelation coefficient 
% ρ(k), k = 1, ..., L, is nonzero. The test statistic is T(T+2) \sum_{k=1}^{L}(\frac{\rho(k)^2}{T-k})
% where T is the sample size, L is the number of autocorrelation lags, and ρ(k) is the sample autocorrelation 
% at lag k. Under the null, the asymptotic distribution of Q is chi-square with L degrees of freedom.

ts_data = iddata(D,[],1);
%opt = arOptions('Approach', 'ls', 'EstCovar', false);
model = ar(ts_data,L,'yw');
ab = model.A;
res = filter(model.A,1,ts_data.y);

[H pValue qstat cvalue] = lbqtest(res);
[acf lags bounds] = autocorr(res);
if(H)
	%disp('Time Series still Dependent');
end
% Ultimately use residuals from mse ||Y_n+1 - [1 Y_n]*[b a]||'
end